@@ -1,4 +1,5 @@
 use crate::draw::Draw;
+use crate::formatted_text::WrapMode;
 use crate::{
     brush::Brush,
     core::{algebra::Vector2, color::Color, math::Rect, pool::Handle},
@@ -13,6 +14,7 @@ use crate::{
     BuildContext, Control, HorizontalAlignment, UINode, UserInterface, VerticalAlignment,
     BRUSH_DARKER, BRUSH_TEXT,
 };
+use clipboard::ClipboardProvider;
 use std::{
     cell::RefCell,
     cmp::{self, Ordering},
@@ -257,6 +259,17 @@ impl<M: MessageData, C: Control<M, C>> TextBox<M, C> {
         }
     }
 
+    fn insert_str(&mut self, str: &str, ui: &UserInterface<M, C>) {
+        let position = self.get_absolute_position(self.caret_position).unwrap_or(0);
+        self.formatted_text.borrow_mut().insert_str(str, position);
+        self.move_caret_x(str.chars().count(), HorizontalDirection::Right, false);
+        ui.send_message(TextBoxMessage::text(
+            self.handle,
+            MessageDirection::ToWidget,
+            self.formatted_text.borrow().text(),
+        ));
+    }
+
     pub fn get_text_len(&self) -> usize {
         self.formatted_text.borrow_mut().get_raw_text().len()
     }
@@ -359,13 +372,13 @@ impl<M: MessageData, C: Control<M, C>> TextBox<M, C> {
         self.formatted_text.borrow().text()
     }
 
-    pub fn set_wrap(&mut self, wrap: bool) -> &mut Self {
+    pub fn set_wrap(&mut self, wrap: WrapMode) -> &mut Self {
         self.formatted_text.borrow_mut().set_wrap(wrap);
         self
     }
 
-    pub fn is_wrap(&self) -> bool {
-        self.formatted_text.borrow().is_wrap()
+    pub fn wrap_mode(&self) -> WrapMode {
+        self.formatted_text.borrow().wrap_mode()
     }
 
     pub fn set_font(&mut self, font: SharedFont) -> &mut Self {
@@ -634,13 +647,12 @@ impl<M: MessageData, C: Control<M, C>> Control<M, C> for TextBox<M, C> {
                         }
                         KeyCode::End => {
                             let text = self.formatted_text.borrow();
+                            let line = &text.get_lines()[self.caret_position.line];
                             if ui.keyboard_modifiers().control {
-                                let line = &text.get_lines()[self.caret_position.line];
                                 self.caret_position.line = text.get_lines().len() - 1;
                                 self.caret_position.offset = line.end - line.begin;
                                 self.selection_range = None;
                             } else if ui.keyboard_modifiers().shift {
-                                let line = &text.get_lines()[self.caret_position.line];
                                 let prev_position = self.caret_position;
                                 self.caret_position.offset = line.end - line.begin;
                                 self.selection_range = Some(SelectionRange {
@@ -651,7 +663,6 @@ impl<M: MessageData, C: Control<M, C>> Control<M, C> for TextBox<M, C> {
                                     },
                                 });
                             } else {
-                                let line = &text.get_lines()[self.caret_position.line];
                                 self.caret_position.offset = line.end - line.begin;
                                 self.selection_range = None;
                             }
@@ -689,6 +700,31 @@ impl<M: MessageData, C: Control<M, C>> Control<M, C> for TextBox<M, C> {
                                 });
                             }
                         }
+                        KeyCode::C if ui.keyboard_modifiers().control => {
+                            if let Some(clipboard) = ui.clipboard_mut() {
+                                if let Some(selection_range) = self.selection_range.as_ref() {
+                                    if let (Some(begin), Some(end)) = (
+                                        self.get_absolute_position(selection_range.begin),
+                                        self.get_absolute_position(selection_range.end),
+                                    ) {
+                                        let _ = clipboard
+                                            .set_contents(String::from(&self.text()[begin..end]));
+                                    }
+                                }
+                            }
+                        }
+                        KeyCode::V if ui.keyboard_modifiers().control => {
+                            if let Some(clipboard) = ui.clipboard_mut() {
+                                if let Ok(content) = clipboard.get_contents() {
+                                    if let Some(selection_range) = self.selection_range {
+                                        self.remove_range(ui, selection_range);
+                                        self.selection_range = None;
+                                    }
+
+                                    self.insert_str(&content, ui);
+                                }
+                            }
+                        }
                         _ => (),
                     },
                     WidgetMessage::GotFocus => {
@@ -791,7 +827,7 @@ pub struct TextBoxBuilder<M: MessageData, C: Control<M, C>> {
     filter: Option<Rc<RefCell<FilterCallback>>>,
     vertical_alignment: VerticalAlignment,
     horizontal_alignment: HorizontalAlignment,
-    wrap: bool,
+    wrap: WrapMode,
     commit_mode: TextCommitMode,
     multiline: bool,
     editable: bool,
@@ -808,7 +844,7 @@ impl<M: MessageData, C: Control<M, C>> TextBoxBuilder<M, C> {
             filter: None,
             vertical_alignment: VerticalAlignment::Top,
             horizontal_alignment: HorizontalAlignment::Left,
-            wrap: false,
+            wrap: WrapMode::NoWrap,
             commit_mode: TextCommitMode::LostFocusPlusEnter,
             multiline: false,
             editable: true,
@@ -850,7 +886,7 @@ impl<M: MessageData, C: Control<M, C>> TextBoxBuilder<M, C> {
         self
     }
 
-    pub fn with_wrap(mut self, wrap: bool) -> Self {
+    pub fn with_wrap(mut self, wrap: WrapMode) -> Self {
         self.wrap = wrap;
         self
     }
